@@ -1,10 +1,10 @@
-/*** databse.c ****************************************************************
+/*** database.c ***************************************************************
 ** 
 ** This file is part of BibTool.
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-2016 Gerd Neugebauer
+** (c) 1996-20120Gerd Neugebauer
 ** 
 ** Net: gene@gerd-neugebauer.de
 ** 
@@ -150,7 +150,7 @@ int apply_modify(db, key, rec)	   		   /*                        */
     { DebugPrint3(SymbolValue(*hp),		   /*                        */
 		  " => ",			   /*                        */
 		  SymbolValue(*(hp+1)));	   /*                        */
-      push_to_record(r, *hp, *(hp+1));		   /*                        */
+      push_to_record(r, *hp, *(hp+1), false);	   /*                        */
     }						   /*			     */
     hp += 2;					   /* Goto next pair.	     */
   }						   /*                        */
