@@ -4,7 +4,7 @@
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-2017 Gerd Neugebauer
+** (c) 1996-2020 Gerd Neugebauer
 ** 
 ** Net: gene@gerd-neugebauer.de
 ** 
@@ -71,6 +71,7 @@ typedef struct rECORD				/*                           */
   Symbol	rc_source;			/* The source of the record. */
  						/*  I.e. the file name it    */
  						/*  has been read from.      */
+  int           rc_lineno;			/* Line number or -1.        */
   struct rECORD *rc_next;			/* Pointer to the next       */
  						/*  record.                  */
   struct rECORD *rc_prev;			/* Pointer to the previous   */
@@ -115,7 +116,6 @@ typedef struct rECORD				/*                           */
 #define RecordSet(R,F)		(RecordFlags(R) |= (F))
 #define RecordClear(R,F)	(RecordFlags(R) &= ~(F))
 
-
 /*-----------------------------------------------------------------------------
 ** Constant:	RecordFlagMARKED
 ** Type:	int
@@ -262,6 +262,17 @@ typedef struct rECORD				/*                           */
 **___________________________________________________			     */
 #define RecordIsMARKED(R)	(RecordFlags(R) & RecordFlagMARKED)
 
+/*-----------------------------------------------------------------------------
+** Macro:	RecordKey()
+** Type:	String 
+** Purpose:	
+**		
+**		
+** Arguments:
+**	R	Record to consider
+**___________________________________________________			     */
+#define RecordKey(R)		(*((R)->rc_heap))
+
 /*-----------------------------------------------------------------------------
 ** Macro:	RecordOldKey()
 ** Type:	String 
@@ -357,6 +368,17 @@ typedef struct rECORD				/*                           */
 **___________________________________________________			     */
 #define RecordSource(R)	((R)->rc_source)
 
+/*-----------------------------------------------------------------------------
+** Macro:	RecordLineno()
+** Type:	int
+** Purpose:	This is the line number where the record has been read from.
+**              The value -1 is used for an unknown line number.
+** Arguments:
+**	R	Record to consider
+** Returns:	
+**___________________________________________________			     */
+#define RecordLineno(R)	((R)->rc_lineno)
+
 /*-----------------------------------------------------------------------------
 ** Macro:	RecordFlags()
 ** Type:	int
@@ -387,7 +409,7 @@ typedef struct rECORD				/*                           */
  void free_1_record _ARG((Record rec));		   /* record.c               */
  void free_record _ARG((Record rec));		   /* record.c               */
  void provide_to_record _ARG((Record rec,Symbol s, Symbol t));/*             */
- void push_to_record _ARG((Record rec,Symbol s, Symbol t));/* record.c       */
+ void push_to_record _ARG((Record rec,Symbol s, Symbol t, bool err));/* record.c*/
  void sort_record _ARG((Record rec));		   /* record.c               */
 
 #endif
