@@ -4,7 +4,7 @@
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-2017 Gerd Neugebauer
+** (c) 1996-2020 Gerd Neugebauer
 ** 
 ** Net: gene@gerd-neugebauer.de
 ** 
@@ -67,7 +67,9 @@ RSC_NEXT('b')
 RSC_NEXT('c')
   RscBoolean( "check.double"	      , r_cd  ,rsc_double_check	  , false   )
   RscBoolean( "check.double.delete"   , r_cdd ,rsc_del_dbl	  , false   )
-  RscByFct(   "check.rule"	      , r_cr  ,add_check_rule(SymbolValue(val)))
+  RscByFct(   "check.rule"	      , r_cr  ,add_check_rule(SymbolValue(val),RULE_NONE))
+  RscByFct(   "check.error.rule"      , r_cer ,add_check_rule(SymbolValue(val),RULE_ERROR))
+  RscByFct(   "check.warning.rule"    , r_cwr ,add_check_rule(SymbolValue(val),RULE_WARNING))
   RscBoolean( "check.case.sensitive"  , r_ccs ,rsc_case_check	  ,  true   )
   RscByFct(   "clear.crossref.map"    , r_ccm ,clear_map()		    )
   RscByFct(   "clear.ignored.words"   , r_ciw ,clear_ignored_words()	    )
@@ -82,7 +84,7 @@ RSC_NEXT('d')
   RscBoolean( "dump.symbols"	      , r_ds  ,rsc_dump_symbols	  , false   )
 RSC_NEXT('e')
   RscString(  "env.separator"	      , r_es  ,rsc_env_sep	  , ENV_SEP ) 
-  RscByFct(   "extract.file"	      , r_ef  ,read_aux(SymbolValue(val),save_input_file,false))
+  RscByFct(   "extract.file"	      , r_ef  ,read_aux(val,save_input_file,false))
   RscByFct(   "extract.regex"	      , r_er  ,save_regex(SymbolValue(val)) )
   RscBoolean( "expand.macros"	      , r_em  ,rsc_expand_macros  , false   )
   RscBoolean( "expand.crossref"	      , r_ex  ,rsc_expand_crossref, false   )
@@ -98,11 +100,11 @@ RSC_NEXT('f')
   RscByFct(   "field.type"	      , r_ft  ,set_symbol_type(SymbolValue(val)))
   RscTerm(    "false"		      , RSC_INIT_FALSE			    )
 RSC_NEXT('i')
-  RscByFct(   "input"		      , r_i   ,save_input_file((char*)SymbolValue(val))  )
+  RscByFct(   "input"		      , r_i   ,save_input_file(val)         )
   RscByFct(   "ignored.word"	      , r_iw  ,add_ignored_word(val)	    )
   RscTerm(    "ilike"		      , RSC_INIT_ILIKE			    )
 RSC_NEXT('k')
-  RscByFct(   "keep.field"            , r_kef ,keep_field(val) )
+  RscByFct(   "keep.field"            , r_kef ,keep_field(val)              )
   RscBoolean( "key.generation"	      , r_kg  ,rsc_make_key	  , false   ) 
   RscByFct(   "key.base"	      , r_kb  ,set_base(SymbolValue(val))   )
   RscByFct(   "key.format"	      , r_kf  ,add_format((char*)SymbolValue(val)))
@@ -112,7 +114,7 @@ RSC_NEXT('k')
 RSC_NEXT('l')
   RscTerm(    "like"		      , RSC_INIT_LIKE			    )
 RSC_NEXT('m')
-  RscByFct(   "macro.file"	      , r_mf  ,save_macro_file((char*)SymbolValue(val)))
+  RscByFct(   "macro.file"	      , r_mf  ,save_macro_file(val))
   RscTerm(    "mod"		      , RSC_INIT_MOD			    )
 RSC_NEXT('n')
   RscByFct(   "new.entry.type"	      , r_net ,def_entry_type(val)	    )
@@ -121,7 +123,7 @@ RSC_NEXT('n')
   RscTerm(    "not"		      , RSC_INIT_NOT			    )
   RscTerm(    "nil"		      , RSC_INIT_NIL			    )
 RSC_NEXT('o')
-  RscByFct(   "output.file"	      , r_of  ,save_output_file((char*)SymbolValue(val)) )
+  RscByFct(   "output.file"	      , r_of  ,save_output_file(val)        )
   RscTerm(    "or"		      , RSC_INIT_OR			    )
 RSC_NEXT('p')
   RscBoolean( "pass.comments"	      , r_pc  ,rsc_pass_comment	  , false   )
@@ -172,12 +174,14 @@ RSC_NEXT('s')
   RscBoolean( "sort.macros"	      , r_sm  ,rsc_srt_macs	  , true    )
   RscBoolean( "sort.reverse"	      , r_sr  ,rsc_sort_reverse   , false   )
   RscByFct(   "sort.order"	      , r_so  ,add_sort_order(val)          )
-  RscByFct(   "sort.format"	      , r_sf  ,add_sort_format((char*)SymbolValue(val))  )
+  RscByFct(   "sort.format"	      , r_sf  ,add_sort_format((char*)SymbolValue(val)))
   RscBoolean( "suppress.initial.newline", r_sin ,rsc_no_nl	  , false   )
-  RscByFct(   "symbol.type"	      , r_st  ,set_symbol_type(SymbolValue(val))         )
+  RscByFct(   "symbol.type"	      , r_st  ,set_symbol_type(SymbolValue(val)))
 RSC_NEXT('t')
-RscByFct(   "tex.define"	      , r_td  ,TeX_def(SymbolValue(val))    ) 
+  RscByFct(   "tex.define"	      , r_td  ,TeX_def(SymbolValue(val))    ) 
   RscTerm(    "true"		      , RSC_INIT_TRUE			    )
+RSC_NEXT('u')
+  RscByFct(   "unique.field"	      , r_uf  ,add_unique_field(val)        ) 
 RSC_NEXT('v')
   RscBoolean( "verbose"		      , r_v   ,rsc_verbose	  , false   ) 
   RscByFct(   "version"		      , r_ver ,show_version()		    ) 
