@@ -4,7 +4,7 @@
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-2017 Gerd Neugebauer
+** (c) 1996-2020 Gerd Neugebauer
 ** 
 ** Net: gene@gerd-neugebauer.de
 ** 
@@ -31,21 +31,13 @@
 #include <bibtool/general.h>
 #include <bibtool/error.h>
 #include <bibtool/io.h>
+#include <bibtool/symbols.h>
 #include "config.h"
 
 /*****************************************************************************/
 /* Internal Programs							     */
 /*===========================================================================*/
 
-#ifdef __STDC__
-#define _ARG(A) A
-#else
-#define _ARG(A) ()
-#endif
- void save_input_file _ARG((char *file));	   /*                        */
- void save_macro_file _ARG((char *file));	   /*                        */
- void save_output_file _ARG((char * file));	   /*                        */
-
 /*****************************************************************************/
 /* External Programs and Variables					     */
 /*===========================================================================*/
@@ -59,15 +51,15 @@
 
 #define InputFilePipeIncrement 8
 
- static char **input_files;
- static int  input_file_size = 0;
- static int  input_file_ptr  = 0;
+ static Symbol* input_files;
+ static int input_file_size = 0;
+ static int input_file_ptr  = 0;
 
 #define InputPipeIsFull		(input_file_ptr >= input_file_size)
 #define InputPipeIsEmpty	(input_file_ptr == 0)
 #define PushToInputPipe(FILE)	input_files[input_file_ptr++] = FILE
 #define ForAllInputFiles(FILE)	for (FILE=input_files;			\
-				     FILE<&input_files[input_file_ptr];	\
+				     FILE<input_files[input_file_ptr];	\
 				     FILE++)
 
 /*-----------------------------------------------------------------------------
@@ -85,25 +77,26 @@
 ** Returns:	nothing
 **___________________________________________________			     */
 void save_input_file(file)			   /*			     */
-  char *file;					   /*			     */
+  Symbol file;					   /*			     */
 {						   /*			     */
-  if (file == NULL)				   /*			     */
+  if (file == NO_SYMBOL)			   /*			     */
   { WARNING("Missing input file name. Flag ignored.");/*		     */
     return;					   /*			     */
   }						   /*			     */
-  if (*file == '-' && file[1] == '\0')		   /*			     */
-  { file = NULL; }				   /*			     */
+  if (*SymbolValue(file) == '-' &&		   /*			     */
+      SymbolValue(file)[1] == '\0')		   /*			     */
+  { file = NO_SYMBOL; }				   /*			     */
 						   /*			     */
   if (InputPipeIsFull)				   /* No space left?	     */
   { input_file_size += InputFilePipeIncrement;	   /*			     */
 						   /*			     */
     if (InputPipeIsEmpty			   /* Try to enlarge array   */
 	? NULL==(input_files=			   /*			     */
-		 (char**)malloc(sizeof(char*)	   /*			     */
+		 (Symbol*)malloc(sizeof(Symbol)	   /*			     */
 				*(size_t)input_file_size))/*		     */
 	: NULL==(input_files=			   /*			     */
-		 (char**)realloc((char*)input_files,/*			     */
-				 sizeof(char*)	   /*			     */
+		 (Symbol*)realloc((char*)input_files,/*			     */
+				 sizeof(Symbol)	   /*			     */
 				 *(size_t)input_file_size))/*		     */
 	)					   /*			     */
     { OUT_OF_MEMORY("input file pipe."); }	   /*			     */
@@ -114,11 +107,10 @@ void save_input_file(file)			   /*			     */
 /*-----------------------------------------------------------------------------
 ** Function:	get_no_inputs()
 ** Type:	int
-** Purpose:	
+** Purpose:	Get the number of input file names stored.
 **		
-** Arguments:
-**		
-** Returns:	
+** Arguments:	none
+** Returns:	the number of currently stored input files
 **___________________________________________________			     */
 int get_no_inputs()				   /*                        */
 {						   /*                        */
@@ -128,17 +120,17 @@ int get_no_inputs()				   /*                        */
 /*-----------------------------------------------------------------------------
 ** Function:	get_input_file()
 ** Type:	char *
-** Purpose:	
+** Purpose:	Getter for a single input file name.
 **		
 ** Arguments:
-**	i	
-** Returns:	
+**	i	the index of the input file to access
+** Returns:	the references input file name or NULL otherwise
 **___________________________________________________			     */
-char * get_input_file(i)			   /*                        */
+Symbol get_input_file(i)			   /*                        */
   int i;					   /*                        */
 { return (i >= 0 && i < input_file_ptr		   /*                        */
 	  ? input_files[i]			   /*                        */
-	  : NULL);	 			   /*                        */
+	  : NO_SYMBOL);	 			   /*                        */
 }						   /*------------------------*/
 
 
@@ -146,7 +138,7 @@ char * get_input_file(i)			   /*                        */
 /***			   Output File Section				   ***/
 /*****************************************************************************/
 
- char *output_file = NULL;		   	   /*			     */
+ static Symbol output_file = NO_SYMBOL;	   	   /*			     */
 
 /*-----------------------------------------------------------------------------
 ** Function:	save_output_file()
@@ -157,22 +149,23 @@ char * get_input_file(i)			   /*                        */
 ** Returns:	nothing
 **___________________________________________________			     */
 void save_output_file(file)			   /*			     */
-  char * file;					   /*			     */
-{ if ( output_file != NULL )			   /*			     */
-  { WARNING2("Output file redefined: ",file); }	   /*			     */
+  Symbol file;					   /*			     */
+{ if (output_file != NO_SYMBOL)			   /*			     */
+  { WARNING2("Output file redefined: ",		   /*			     */
+	     SymbolValue(file));		   /*			     */
+  }						   /*			     */
   output_file = file;				   /*			     */
 }						   /*------------------------*/
 
 /*-----------------------------------------------------------------------------
 ** Function:	get_output_file()
-** Type:	char*
-** Purpose:	
+** Type:	Symbol
+** Purpose:	This is the getter for the output file name.
 **		
-** Arguments:
-**		
-** Returns:	
+** Arguments:	none
+** Returns:	the output file name or NULL for STDOUT
 **___________________________________________________			     */
-char* get_output_file()			   	   /*			     */
+Symbol get_output_file()		   	   /*			     */
 { return output_file;				   /*			     */
 }						   /*------------------------*/
 
@@ -181,7 +174,7 @@ char* get_output_file()			   	   /*			     */
 /***			   Macro File Section				   ***/
 /*****************************************************************************/
 
- char *macro_file = NULL;		   	   /*			     */
+ static Symbol macro_file = NO_SYMBOL;	   	   /*			     */
 
 /*-----------------------------------------------------------------------------
 ** Function:	save_macro_file()
@@ -192,21 +185,21 @@ char* get_output_file()			   	   /*			     */
 ** Returns:	nothing
 **___________________________________________________			     */
 void save_macro_file(file)			   /*			     */
-  char *file;					   /*			     */
+  Symbol file;					   /*			     */
 { if ( macro_file != NULL )			   /*			     */
-  { WARNING2("Macro file redefined: ",file); }	   /*			     */
+  { WARNING2("Macro file redefined: ",		   /*			     */
+	     SymbolValue(file));		   /*			     */
+  }						   /*			     */
   macro_file = file;				   /*			     */
 }						   /*------------------------*/
 
 /*-----------------------------------------------------------------------------
 ** Function:	get_macro_file()
-** Type:	char*
-** Purpose:	
+** Purpose:	This is the getter for the macro file name.
 **		
-** Arguments:
-**		
-** Returns:	
+** Arguments:	none
+** Returns:	the macro file name or NULL for none
 **___________________________________________________			     */
-char* get_macro_file()			   	   /*			     */
+Symbol get_macro_file()			   	   /*			     */
 { return macro_file;				   /*			     */
 }						   /*------------------------*/
