@@ -4,7 +4,7 @@
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-2017 Gerd Neugebauer
+** (c) 1996-2020 Gerd Neugebauer
 ** 
 ** Net: gene@gerd-neugebauer.de
 ** 
@@ -64,6 +64,7 @@
 #include <bibtool/bibtool.h>
 #include <bibtool/sbuffer.h>
 #include <bibtool/crossref.h>
+#include <bibtool/check.h>
 #include <bibtool/io.h>
 #ifdef HAVE_LIBKPATHSEA
 #ifdef __STDC__
@@ -83,7 +84,6 @@
 #define _ARG(A) ()
 #endif
  int main _ARG((int argc,char *argv[]));	   /* main.c                 */
- static bool dbl_check _ARG((DB db,Record rec));   /* main.c                 */
  static bool do_keys _ARG((DB db,Record rec));	   /* main.c                 */
  static bool do_no_keys _ARG((DB db,Record rec));  /* main.c                 */
  static bool update_crossref _ARG((DB db,Record rec));/* main.c              */
@@ -153,7 +153,7 @@ char * getenv(name)				   /*			     */
     "\t%c$\t\tSymbol table output (debugging only)\n",
 #endif
     0L,
-    "Copyright (C) 2016 Gerd Neugebauer",
+    "Copyright (C) 2019 Gerd Neugebauer",
     "gene@gerd-neugebauer.de"
   };
 
@@ -215,12 +215,12 @@ static void usage(fullp)			   /*			     */
   ErrPrintF("%s\n",SPECIAL_OPTIONS);		   /*                        */
 }						   /*------------------------*/
 
-#define UnknownWarning(X)     WARNING2("Unknown flag ignored: ",X)
+#define UnknownWarning(X)     WARNING2("Unknown flag ignored: ", X)
 #define NoSFileWarning	      WARNING("Missing select option. Flag ignored.")
-#define NoFileError(X)	      WARNING3("File ",X," not found.")
+#define NoFileError(X)	      WARNING3("File ", SymbolValue(X), " not found.")
 #define MissingPattern	      WARNING("Missing pattern.")
 #define MissingResource	      WARNING("Missing resource.")
-#define NoRscError(X)	      WARNING3("Resource file ",X," not found.")
+#define NoRscError(X)	      WARNING3("Resource file ", X, " not found.")
 
 
 /*****************************************************************************/
@@ -324,25 +324,68 @@ static void write_macros(m_file, the_db)	   /*                        */
 
 /*-----------------------------------------------------------------------------
 ** Function*:	read_in_files()
-** Type:	static void
-** Purpose:	
-**		
+** Type:	void
+** Purpose:	Read all the files in the input file pipe.
+**		Note that additional files can be pushed during this
+**		function is active.
 ** Arguments:
 **	db	the database
 ** Returns:	nothing
 **___________________________________________________			     */
 static void read_in_files(db)		   	   /*                        */
   DB db;					   /*                        */
-{ int n = get_no_inputs();			   /*                        */
-  int i;					   /*                        */
-  for (i = 0; i < n; i++)		   	   /* For all input files    */
-  { if (read_db(db,			   	   /*                        */
-		(String)get_input_file(i),	   /*                        */
-		rsc_verbose))	   		   /*                        */
-    { NoFileError(get_input_file(i)); }		   /*			     */
+{ int i;					   /*                        */
+  Symbol in;					   /*                        */
+						   /*                        */
+  for (i = 0; i < get_no_inputs(); i++)	   	   /* For all input files    */
+  { in = get_input_file(i);			   /*			     */
+    if (read_db(db, SymbolValue(in), rsc_verbose)) /*                        */
+    { NoFileError(in); }			   /*			     */
   }						   /*			     */
 }						   /*------------------------*/
 
+/*-----------------------------------------------------------------------------
+** Function*:	write_output()
+** Type:	void
+** Purpose:	
+**		
+** Arguments:
+**	db	the database
+** Returns:	nothing
+**___________________________________________________			     */
+static void write_output(db)		   	   /*                        */
+  DB db;					   /*                        */
+{ FILE  *file;				   	   /*                        */
+  Symbol o_file = get_output_file();		   /*                        */
+ 						   /*                        */
+  if (o_file == NO_SYMBOL)		   	   /*                        */
+  { file = stdout; }				   /*                        */
+  else if (*o_file == '\0')		   	   /*                        */
+  { return; }					   /*                        */
+  else if ((file=fopen((char*)SymbolValue(o_file),"w")) == NULL)/*           */
+  { file = stdout;				   /*                        */
+  }						   /*                        */
+ 						   /*                        */
+  if (rsc_select) { rsc_del_q = false; }	   /*                        */
+ 						   /*                        */
+  { char * print_spec = (char*)rsc_print_et;	   /*                        */
+ 						   /*                        */
+    if (rsc_expand_macros)			   /*                        */
+    { char * cp;				   /*                        */
+      print_spec = new_string(print_spec);	   /*                        */
+      for (cp=print_spec; *cp; cp++)		   /*                        */
+      { if (*cp == 's' ||			   /*                        */
+	    *cp == 'S' ||			   /*                        */
+	    *cp == '$' ) *cp = ' ';		   /*                        */
+      }						   /*                        */
+    }						   /*                        */
+    print_db(file, db, print_spec);	   	   /*                        */
+    if (rsc_expand_macros) free(print_spec);	   /*                        */
+  }						   /*                        */
+ 						   /*                        */
+  if (file != stdout) { fclose(file); }	   	   /*                        */
+}						   /*------------------------*/
+
 #define Toggle(X) X = !(X)
 
 /*-----------------------------------------------------------------------------
@@ -370,12 +413,10 @@ int main(argc,argv)				   /*			     */
 						   /* contains a reference to*/
 						   /* this database.	     */
   int	i;				   	   /*			     */
-  FILE	*file;				   	   /*                        */
   bool	need_rsc = true;		   	   /*			     */
   int	(*fct)();			   	   /* Function pointer	     */
   int	c_len;					   /*                        */
   int   *c = NULL;				   /*                        */
-  char  *o_file;				   /*                        */
  						   /*                        */
   init_error(stderr);				   /*                        */
   init_bibtool(argv[0]);			   /*                        */
@@ -383,7 +424,7 @@ int main(argc,argv)				   /*			     */
   for (i = 1; i < argc; i++)			   /*			     */
   { char *ap;				   	   /*			     */
     if (*(ap=argv[i]) != OptionLeadingCharacter)   /*			     */
-    { save_input_file(argv[i]);			   /*			     */
+    { save_input_file(symbol((String)argv[i]));	   /*			     */
     }						   /*			     */
     else					   /*			     */
     { switch (*++ap)				   /*			     */
@@ -396,18 +437,18 @@ int main(argc,argv)				   /*			     */
 	case 'f': add_format(argv[++i]);	   /*	!!! no break !!!     */
 	case 'F': rsc_make_key = true;	    break; /* key generation	     */
 	case 'h': usage(true); return 1;	   /* print help	     */
-	case 'i': save_input_file(argv[++i]); break;/*			     */
+	case 'i': save_input_file(symbol((String)argv[++i])); break;/*       */
 	case 'K': add_format("long");	    break; /* key generation	     */
 	case 'k': add_format("short");	    break; /* key generation	     */
 	case 'M':				   /* print macro table	     */
 	case 'm': rsc_all_macs = (*ap=='m');	   /* print macro table	     */
-	  save_macro_file(argv[++i]);		   /*			     */
+	  save_macro_file(symbol((String)argv[++i]));/*			     */
 	  break;				   /*			     */
 	case 'o':				   /* output file	     */
 	  if (++i < argc)   			   /*		             */
-	  { save_output_file(argv[i]); }	   /*                        */
+	  { save_output_file(symbol((String)argv[i])); }/*                   */
 	  else					   /*                        */
-	  { WARNING("Missing output file name"); } /*                        */
+	  { WARNING((String)"Missing output file name"); } /*                */
 	  break; 				   /*			     */
 	case 'q': Toggle(rsc_quiet);	    break; /* quiet		     */
 	case 'r':				   /* resource file	     */
@@ -460,7 +501,7 @@ int main(argc,argv)				   /*			     */
   if (need_rsc) { (void)search_rsc(); }	   	   /*			     */
 						   /*			     */
   if (get_no_inputs() == 0)			   /* If no input file given */
-  { save_input_file("-"); }			   /*  then read from stdin  */
+  { save_input_file(symbol((String)"-")); }	   /*  then read from stdin  */
 						   /*			     */
   init_read();					   /* Just in case the path  */
  						   /*  has been modified.    */
@@ -490,11 +531,13 @@ int main(argc,argv)				   /*			     */
     db_forall(the_db,expand_crossref);		   /*                        */
   }						   /*			     */
 						   /*			     */
-  if (rsc_make_key)				   /*                        */
-  {						   /*                        */
+  if (rsc_sort || rsc_make_key || need_sort_key) { /*                        */
     DebugPrint1("start keygen");		   /*                        */
     start_key_gen();				   /*                        */
+  }						   /*                        */
 						   /*			     */
+  if (rsc_make_key)		   		   /*                        */
+  {						   /*                        */
     if (rsc_key_preserve)	   		   /*                        */
     { db_forall(the_db, mark_key); }		   /*                        */
 						   /*			     */
@@ -525,45 +568,21 @@ int main(argc,argv)				   /*			     */
     { if (rsc_sort_reverse) fct = rec_lt;	   /*                        */
       else		    fct = rec_gt;	   /*                        */
     }						   /*                        */
-    db_sort(the_db,fct);			   /*                        */
+    db_sort(the_db, fct);			   /*                        */
   }						   /*                        */
  						   /*                        */
   if (rsc_srt_macs)		   	   	   /* Maybe sort macros      */
   { db_mac_sort(the_db); }			   /*                        */
  						   /*                        */
-  if (rsc_double_check)		   	   	   /* Maybe look for doubles */
-  { db_forall(the_db,dbl_check); }		   /*                        */
+  apply_checks(the_db);				   /*                        */
  						   /*                        */
-  o_file = get_output_file();			   /*                        */
- 						   /*                        */
-  if (o_file == NULL ||			   	   /*                        */
-      (file=fopen(o_file,"w")) == NULL)	   	   /*                        */
-  { file = stdout; }				   /*                        */
- 						   /*                        */
-  if (rsc_select) { rsc_del_q = false; }	   /*                        */
- 						   /*                        */
-  { char * print_spec = (char*)rsc_print_et;	   /*                        */
- 						   /*                        */
-    if (rsc_expand_macros)			   /*                        */
-    { char * cp;				   /*                        */
-      print_spec = new_string(print_spec);	   /*                        */
-      for (cp=print_spec; *cp; cp++)		   /*                        */
-      { if (*cp == 's' ||			   /*                        */
-	    *cp == 'S' ||			   /*                        */
-	    *cp == '$' ) *cp = ' ';		   /*                        */
-      }						   /*                        */
-    }						   /*                        */
-    print_db(file,the_db,print_spec);	   	   /*                        */
-    if (rsc_expand_macros) free(print_spec);	   /*                        */
-  }						   /*                        */
- 						   /*                        */
-  if (file != stdout) { fclose(file); }	   	   /*                        */
+  write_output(the_db);				   /*                        */
 						   /*			     */
   write_macros(get_macro_file(), the_db);	   /*                        */
 						   /*			     */
   if (rsc_cnt_all || rsc_cnt_used)		   /*			     */
   { int i;					   /*                        */
-    int *cnt = db_count(the_db,(int*)NULL);	   /*                        */
+    int *cnt = db_count(the_db, (int*)NULL);	   /*                        */
 						   /*                        */
     ErrC('\n');		   	   	   	   /*			     */
     for (i = 0; i < c_len; ++i)			   /*			     */
@@ -674,7 +693,7 @@ static bool do_keys(db, rec)			   /*                        */
   rewrite_record(db, rec);			   /*			     */
   sort_record(rec);				   /*                        */
   make_key(db,rec);				   /*                        */
-  if (rsc_sort || rsc_double_check)		   /*                        */
+  if (rsc_sort || rsc_double_check || need_sort_key)/*                       */
   { make_sort_key(db,rec); }		   	   /*                        */
   return false;					   /*                        */
 }						   /*------------------------*/
@@ -694,7 +713,7 @@ static bool do_no_keys(db, rec)			   /*                        */
   rewrite_record(db, rec);			   /*			     */
   sort_record(rec);				   /*                        */
   mark_key(db,rec);				   /*                        */
-  if (rsc_sort || rsc_double_check)		   /*                        */
+  if (rsc_sort || rsc_double_check || need_sort_key)/*                       */
   { make_sort_key(db,rec); }		   	   /*                        */
   return false;					   /*                        */
 }						   /*------------------------*/
@@ -752,46 +771,6 @@ static bool update_crossref(db, rec)		   /*			     */
   return false;					   /*                        */
 }						   /*------------------------*/
 
-#define equal_records(R1,R2) RecordSortkey(R1) == RecordSortkey(R2)
-
-/*-----------------------------------------------------------------------------
-** Function*:	dbl_check()
-** Purpose:	
-**		
-**
-** Arguments:
-**	rec	teh record
-** Returns:	|false|
-**___________________________________________________			     */
-static bool dbl_check(db, rec)			   /*                        */
-  DB	 db;					   /*                        */
-  Record rec;					   /*                        */
-{						   /*                        */
-  if (PrevRecord(rec) != RecordNULL		   /*                        */
-      && equal_records(PrevRecord(rec),rec))	   /*			     */
-  {						   /*                        */
-    if (!rsc_quiet)				   /*                        */
-    { Symbol k1 = *RecordHeap(rec);		   /*                        */
-      Symbol k2 = *RecordHeap(PrevRecord(rec));	   /*                        */
-      ErrPrint("*** BibTool WARNING: Possible double entries discovered: \n***\t");
-      if (k1 == NO_SYMBOL) k1 = sym_empty;	   /*                        */
-      if (k2 == NO_SYMBOL) k2 = sym_empty;	   /*                        */
-      ErrPrint((char*)SymbolValue(k2));		   /*                        */
-      ErrPrint(" =?= ");			   /*                        */
-      ErrPrint((char*)SymbolValue(k1));		   /*                        */
-      ErrPrint("\n***\t");			   /*                        */
-      ErrPrint((char*)SymbolValue(RecordSortkey(rec)));/*		     */
-      ErrPrint("\n");				   /*                        */
-    }						   /*                        */
-    if (rsc_del_dbl)				   /*                        */
-    { delete_record(db,rec); }		   	   /*                        */
-    else 					   /*                        */
-    { SetRecordDELETED(rec); }			   /*                        */
-  }						   /*			     */
- 						   /*                        */
-  return false;					   /*                        */
-}						   /*------------------------*/
-
 #ifdef DEBUG
 /*-----------------------------------------------------------------------------
 ** Function*:	printchar()
