@@ -4,7 +4,7 @@
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-2016 Gerd Neugebauer
+** (c) 1996-2020 Gerd Neugebauer
 ** 
 ** Net: gene@gerd-neugebauer.de
 ** 
@@ -49,7 +49,7 @@
  void free_1_record _ARG((Record rec)); 	   /* record.c               */
  void free_record _ARG((Record rec)); 		   /* record.c               */
  void provide_to_record _ARG((Record rec,Symbol s,Symbol t));/* record.c     */
- void push_to_record _ARG((Record rec,Symbol s,Symbol t));/* record.c        */
+ void push_to_record _ARG((Record rec,Symbol s,Symbol t, bool err));/* record.c*/
  void sort_record _ARG((Record rec)); 		   /* record.c               */
 
 /*****************************************************************************/
@@ -86,11 +86,15 @@ Record copy_record(rec)				   /*			     */
   RecordFree(new)	  = RecordFree(rec);	   /*			     */
   RecordComment(new)	  = RecordComment(rec);	   /*			     */
   RecordSource(new)	  = RecordSource(rec);	   /*			     */
+  if (RecordSource(new)) {			   /*			     */
+    LinkSymbol(RecordSource(new));		   /*			     */
+  }			  			   /*			     */
+  RecordLineno(new)	  = RecordLineno(rec);	   /*			     */
   RecordHeap(new)	  = new_heap;		   /*			     */
   RecordFlags(new)	  = RecordFlags(rec);	   /*			     */
   for (i = 0, old_heap = RecordHeap(rec);	   /*			     */
-	i < RecordFree(new);			   /*			     */
-	++i)					   /*			     */
+       i < RecordFree(new);			   /*			     */
+       ++i)					   /*			     */
   { *(new_heap++) = *(old_heap++); }		   /*			     */
   return (new);					   /*			     */
 }						   /*------------------------*/
@@ -125,6 +129,7 @@ Record new_record(token,size)			   /*			     */
   RecordFree(new)	= size;			   /*			     */
   RecordComment(new)	= sym_empty;		   /*			     */
   RecordSource(new)	= sym_empty;		   /*			     */
+  RecordLineno(new)	= -1;             	   /*			     */
   RecordFlags(new)	= 0;	   		   /*			     */
   RecordHeap(new)	= new_heap;		   /*			     */
   for (i = 0; i < RecordFree(new); ++i)		   /*			     */
@@ -316,14 +321,17 @@ int count_record(rec)				   /*                        */
 **		copies and it is possible to avoid expensive string
 **		comparisons. 
 ** Arguments:
+**	rec	the record
 **	s	Left hand side of the equation.
 **	t	Right hand side of the equation.
+**	err	indicator that a warning for double fields should be issued.
 ** Returns:	nothing
 **___________________________________________________			     */
-void push_to_record(rec, s, t)			   /*			     */
+void push_to_record(rec, s, t, err)		   /*			     */
   register Record rec;				   /*                        */
   register Symbol s;				   /*			     */
   register Symbol t;				   /*			     */
+  bool err;					   /*			     */
 { register int i;		   		   /*			     */
    						   /*                        */
   if (s == sym_crossref || s == sym_xdata)	   /*                        */
@@ -333,6 +341,12 @@ void push_to_record(rec, s, t)			   /*			     */
   { if (RecordHeap(rec)[i] == s)		   /* if found then          */
     { sym_unlink(RecordHeap(rec)[i + 1]);	   /*                        */
       RecordHeap(rec)[i + 1] = t;		   /* overwrite the value    */
+      if (err) {				   /*			     */
+	error(ERR_WARNING|ERR_FILE,		   /*                        */
+	      (String)"Duplicate field `", s,	   /*                        */
+	      (String)"' overwritten", NULL, 0,	   /*                        */
+	      RecordLineno(rec), RecordSource(rec));/*                       */
+      }						   /*			     */
       return;					   /*                        */
     }   					   /*                        */
   }						   /*                        */
