@@ -4,7 +4,7 @@
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-2017 Gerd Neugebauer
+** (c) 1996-2020 Gerd Neugebauer
 ** 
 ** Net: gene@gerd-neugebauer.de
 ** 
@@ -42,13 +42,15 @@
 #include <bibtool/print.h>
 #include <bibtool/tex_aux.h>
 #include <bibtool/tex_read.h>
+#include <bibtool/check.h>
+#include <bibtool/crossref.h>
 #include <bibtool/key.h>
 #include <bibtool/rewrite.h>
 #include <bibtool/s_parse.h>
 #include <bibtool/entry.h>
 #include <bibtool/type.h>
-#include <bibtool/crossref.h>
 #include <bibtool/version.h>
+#include <bibtool/io.h>
 #include "config.h"
 #define RSC_INIT
 #include <bibtool/rsc.h>
@@ -71,10 +73,6 @@
 /* External Programs							     */
 /*===========================================================================*/
 
- extern void save_input_file _ARG((char *file));   /* main.c		     */
- extern void save_macro_file _ARG((char * file));  /* main.c		     */
- extern void save_output_file _ARG((char * file)); /* main.c		     */
-
 #ifndef __STDC__
 #ifndef HAVE_GETENV
  extern char * getenv _ARG((char* name));	   /*			     */
