@@ -4,7 +4,7 @@
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-2017 Gerd Neugebauer
+** (c) 1996-2020 Gerd Neugebauer
 ** 
 ** Net: gene@gerd-neugebauer.de
 ** 
@@ -140,7 +140,8 @@
  Symbol sym_open_brace	  = NO_SYMBOL;		   /*			     */
  Symbol sym_close_brace	  = NO_SYMBOL;		   /*			     */
  Symbol sym_et		  = NO_SYMBOL;		   /*			     */
-
+ Symbol sym_key		  = NO_SYMBOL;		   /*			     */
+ Symbol sym_sortkey	  = NO_SYMBOL;		   /*			     */
 
 /*****************************************************************************/
 /***			Misc string allocation routine			   ***/
@@ -266,8 +267,12 @@ void init_symbols()				   /*			     */
   sym_open_brace   = symbol((String)"{");	   /*                        */
   sym_close_brace  = symbol((String)"}");	   /*                        */
   sym_et	   = symbol((String)"&");	   /*                        */
+  sym_key	   = symbol((String)"$key");	   /*                        */
+  sym_sortkey	   = symbol((String)"$sortkey");   /*                        */
 }						   /*------------------------*/
 
+#if defined(DEBUG) || defined(COMPLEX_SYMBOL)
+
 /*-----------------------------------------------------------------------------
 ** Function*:	get_sym_tab()
 ** Type:	SymTab
@@ -296,6 +301,8 @@ static SymTab* get_sym_tab(sym)			   /*			     */
   return NULL;			   		   /*			     */
 }						   /*------------------------*/
 
+#endif
+
 /*-----------------------------------------------------------------------------
 ** Function:	symbol()
 ** Purpose:	Add a symbol to the global symbol table. If the string
@@ -414,17 +421,17 @@ void sym_del(sym)				   /*                        */
   sym_pipe[sym_pipe_ptr] = sym;			   /*                        */
   if ( ++sym_pipe_ptr < SYM_PIPE_SIZE) return;	   /*                        */
  						   /*                        */
-  while (--sym_pipe_ptr > 0)
-  { sym = sym_pipe[sym_pipe_ptr];
-    if (SymbolCount(sym) > 0) continue;
-
-    stp = get_sym_tab(sym);
-    if (stp == NULL) continue;
-    st	 = *stp;
-    *stp = NextSymTab(st);
-    free(st);
-    free(SymbolValue(sym));
-    free(sym);
+  while (--sym_pipe_ptr > 0)			   /*                        */
+  { sym = sym_pipe[sym_pipe_ptr];		   /*                        */
+    if (SymbolCount(sym) > 0) continue;		   /*                        */
+						   /*                        */
+    stp = get_sym_tab(sym);			   /*                        */
+    if (stp == NULL) continue;			   /*                        */
+    st	 = *stp;				   /*                        */
+    *stp = NextSymTab(st);			   /*                        */
+    free(st);					   /*                        */
+    free(SymbolValue(sym));			   /*                        */
+    free(sym);					   /*                        */
   }						   /*                        */
 #endif
 }						   /*------------------------*/
