@@ -4,7 +4,7 @@
 ** It is distributed under the GNU General Public License.
 ** See the file COPYING for details.
 ** 
-** (c) 1996-2017 Gerd Neugebauer
+** (c) 1996-2020 Gerd Neugebauer
 ** 
 ** Net: gene@gerd-neugebauer.de
 ** 
@@ -161,12 +161,12 @@ bool aux_used(s)				   /*                        */
 **___________________________________________________			     */
 bool read_aux(fname, fct, verbose)		   /*                        */
   String 	fname;			   	   /* aux file name          */
-  void		(*fct)_ARG((char*));		   /*                        */
+  void		(*fct)_ARG((Symbol));		   /*                        */
   bool          verbose;			   /*                        */
 { FILE 	        *file;				   /*                        */
   int           c;                                 /*                        */
-  String	s;				   /*                        */
- 						   /*                        */
+  char*		s;				   /*                        */
+  	  	  	  	  	  	   /*                        */
   cite_star  = false;				   /*                        */
   rsc_select = true; 				   /*                        */
   					   	   /*                        */
@@ -175,8 +175,12 @@ bool read_aux(fname, fct, verbose)		   /*                        */
     sbputs((char*)fname,sb);			   /*                        */
     sbputs(".aux",sb);                             /*                        */
     file = fopen(sbflush(sb),"r");                 /*                        */
+    if (file != NULL && verbose)	   	   /*                        */
+    { VerbosePrint2("Analyzing ", sbflush(sb)); }  /*                        */
     (void)sbclose(sb);				   /*                        */
-  }			   			   /*                        */
+  } else if (verbose) {	   			   /*                        */
+    VerbosePrint2("Analyzing ", fname); 	   /*                        */
+  }						   /*                        */
   if (file == NULL)				   /*                        */
   { ERROR3("aux file ",fname," not found.");	   /*                        */
     return true;				   /*                        */
@@ -184,19 +188,16 @@ bool read_aux(fname, fct, verbose)		   /*                        */
  						   /*                        */
   rsc_del_q = false;				   /*                        */
  						   /*                        */
-  if (verbose) 				   	   /*                        */
-  { VerbosePrint2("Analyzing ", fname); }	   /*                        */
- 						   /*                        */
   if (aux_sb == (StringBuffer*)0)		   /*                        */
   { aux_sb = sbopen(); }			   /*                        */
  						   /*                        */
-  for(c = getc(file); c != EOF; c = getc(file))    /*                        */
+  for (c = getc(file); c != EOF; c = getc(file))   /*                        */
   { if (c == '\\') 				   /*                        */
-    { for(c = getc(file);                          /*                        */
-          c != EOF && (is_alpha(c&0xff) || c == '@');/*                      */
-          c = getc(file))			   /*                        */
+    { for (c = getc(file);                         /*                        */
+           c != EOF && (is_alpha(c&0xff) || c == '@');/*                     */
+           c = getc(file))			   /*                        */
       { (void)sbputchar(c, aux_sb); }              /*                        */
-      s = (String)sbflush(aux_sb);		   /*                        */
+      s = sbflush(aux_sb);			   /*                        */
       sbrewind(aux_sb);				   /*                        */
  						   /*                        */
       if (strcmp((char*)s, "citation") == 0 )	   /*                        */
@@ -205,7 +206,7 @@ bool read_aux(fname, fct, verbose)		   /*                        */
 	  { case EOF: break;			   /*                        */
 	    case ',':				   /*                        */
 	    case '}':		   	   	   /*                        */
-	      s = (String)sbflush(aux_sb);	   /*                        */
+	      s = sbflush(aux_sb);		   /*                        */
 	      sbrewind(aux_sb);			   /*                        */
 	      save_ref((String)s);		   /*                        */
 	      break;				   /*                        */
@@ -220,9 +221,9 @@ bool read_aux(fname, fct, verbose)		   /*                        */
 	while ( c != '}' && c != EOF )		   /*                        */
 	{ c = getc(file);			   /*                        */
 	  if ( c == ',' || c == '}' )		   /*                        */
-	  { s = (String)sbflush(aux_sb);	   /*                        */
+	  { s = sbflush(aux_sb);		   /*                        */
 	    sbrewind(aux_sb);			   /*                        */
-	    (*fct)((char*)s);	   	   	   /*                        */
+	    (*fct)(symbol((String)s));	   	   /*                        */
 	  }					   /*                        */
 	  else					   /*                        */
 	  { (void)sbputchar(c, aux_sb); }	   /*                        */
@@ -231,16 +232,19 @@ bool read_aux(fname, fct, verbose)		   /*                        */
       else if (strcmp((char*)s, "@input") == 0 )   /* Read another aux file  */
       { while ((c=getc(file)) != '}' && c != EOF)  /*                        */
 	{ (void)sbputchar(c,aux_sb); }		   /*                        */
-	s = (String)sbflush(aux_sb);		   /*                        */
+	s = sbflush(aux_sb);			   /*                        */
 	sbrewind(aux_sb);			   /*                        */
 	    					   /*                        */
-	read_aux(s, fct, verbose);	   	   /*                        */
+	read_aux((String)s, fct, verbose);   	   /*                        */
+      }						   /*                        */
+      else if (strcmp(s, "endinput") == 0 )	   /* At end                 */
+      { break;					   /*                        */
       }						   /*                        */
     }						   /*                        */
   }						   /*                        */
  						   /*                        */
   (void)fclose(file);				   /*                        */
- 						   /*                        */
+						   /*                        */
 #ifdef DEBUG
   { register int i;				   /*                        */
     WordList wl;				   /*			     */
@@ -313,7 +317,7 @@ bool apply_aux(db)				   /*                        */
       Record r = rec;				   /*                        */
  						   /*                        */
       for (count = rsc_xref_limit;		   /* Prevent infinite loop  */
-	   count >= 0	      &&		   /*                        */
+	   count >= 0	     && 		   /*                        */
 	     RecordIsXREF(r) &&		   	   /*                        */
 	     !RecordIsDELETED(r);		   /*                        */
 	   count-- )				   /*                        */
